"""placeholder

Revision ID: 22fb1af2d51e
Revises: 57539722e5cf
Create Date: 2019-12-03 00:56:06.023042

"""

# revision identifiers, used by Alembic.
revision = '22fb1af2d51e'
down_revision = '57539722e5cf'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
